import java.awt.*;
import java.util.*;
import java.awt.event.*;
import java.awt.image.*;
import java.awt.geom.*;


////////////////////
// COMPILATION LINE
//-cp .;vbp.jar;QTJava.zip $BaseName

////////////////////
// RUN LINE
//-classpath .;vbp.jar;QTJava.zip $FileName

public  class slideGame extends Frame {
    
    // window sizers
    static int kWidth ;
    static int kHeight ;
    
    // Image handlers and GUI
    static PixelSource ps;
    static ImageWrangler iw;
    static slideGame slideGUI;
    static Image myImage;
    static GraphicsDevice myDevice;
    
    // our looping boolean
    static boolean running = true;
    
    // size of the chunks
    static int tileSize = 40;
    static ArrayList vtiles = new ArrayList();
    static int blankX = 0;
    static int blankY = 0;
    
    // constructor
    slideGame() {
        kWidth = 640;
        kHeight = 480;
        
        // list out the divisors of 640 and 480
/*        for (int i = 1; i < 640; i++) {
            
            if ( (640 % i == 0) && (480 % i == 0) ) {
                System.out.println("Common divisor: " + i);
            }
        }
        
        System.exit(5);
        */
    }
    
    // main function
    public static void main(String args[]) {
        
        // create and size the window
        slideGUI = new slideGame();
       	slideGUI.setSize(kWidth + 100, kHeight + 100);
        
       	// set the visibility and layout of the window
        slideGUI.show();
        slideGUI.toFront();
        slideGUI.setLayout (null);
        
        // get our video source
        ps = new PixelSource(kWidth,kHeight);
        
        // get our image handler
        iw = new ImageWrangler(ps.vidWidth,kHeight,ps.getMasks());
        
        //add a listener for shutting the window, give it a method to call (thisWindowClosing)
        slideGUI.addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent e) {
                slideGUI.thisWindowClosing(e);
            }
        });
 
        // setup the video Tiles
        for (int x = 0; x < (kWidth / tileSize); x++) {
            for (int y = 0; y < (kHeight / tileSize); y++) {
                
                // don't add the top corner tile
                if ( (x > 0) || (y > 0)) {
                    // setup new tiles
                    videoTile vt = new videoTile(x, y, tileSize);
                    
                    // add the tile to the arraylist
                    vtiles.add(vt);
                }
            }
        }
        
        // always loop
        while (running) {
            
            // grab a frame
            LookAtAFrame();
            
            // paint the frame
            slideGUI.repaint();
        }
    }
    
    
    
    static void LookAtAFrame() {
        
        ps.grabFrame();
        myImage = iw.imageFromArray(ps.getPixelArray());
        
    }
    
    public void update(Graphics g){
        paint(g);
    }
    
    public void paint(Graphics g) {

        Graphics2D g2 = (Graphics2D)g;
        int rgb[] = new int[4];

        // clear the screen
        g2.setColor(new Color(255, 255, 255));
        g2.fillRect(50, 50, kWidth + 50 - 1, kHeight + 50 - 1);

        // set the screen composite mode
//        Composite c = AlphaComposite.getInstance(AlphaComposite.SRC_OVER, .7f);
//        g2.setComposite(c);

        videoTile currentTile;
        
        // decide which tile will be moving
        int shiftX = -1;
        int shiftY = -1;
        
        double randDirection = Math.random();
        
        do {
            double randVert = Math.random();
            double randHor = Math.random();
            if (randDirection < .5) {
                // shift up and down
                shiftX = blankX;
                
                // figure out which direction to shift in
                if (randVert < .5) {
                    shiftY = blankY - 1;
                } else {
                    shiftY = blankY + 1;
                }
            } else {
                
                // shift left and right
                shiftY = blankY;
                
                // figure out which direction to shift in
                if (randHor < .5) {
                    shiftX = blankX - 1;
                } else {
                    shiftX = blankX + 1;
                }
            }
        } while ( ((shiftX < 0) || (shiftX >= (kWidth / tileSize))) && ((shiftY < 0) || (shiftY >= (kHeight / tileSize))) );
        
        // now shiftX and shiftY hold the tile we are looking for
        
        // loop through and draw the tiles
        for (int i = 0; i < vtiles.size(); i++) {
                        
            // get the tile
            currentTile = (videoTile)vtiles.get(i);
                    
            int vsx = currentTile.getSourceX();
            int vsy = currentTile.getSourceY();
            int vdx = currentTile.getDestX();
            int vdy = currentTile.getDestY();

//            System.out.println("SX: " + vsx + " SY: " + vsy + " DX: " + vdx + " DY: " + vdy);
            
            // draw the image from myImage to the graphics object
            g2.drawImage(myImage, vdx + 50 , vdy + 50, vdx + tileSize + 50, vdy + tileSize + 50, vsx, vsy, vsx + tileSize, vsy + tileSize, null);
        
            // check and see if this is the tile that will be moving
            if ( (currentTile.xTilePosition == shiftX) && (currentTile.yTilePosition == shiftY)) {
                
                // tell the tile which direction to shift in
                if (shiftX < blankX) {
                    currentTile.shiftRight();
                }
                
                if (shiftX > blankX) {
                    currentTile.shiftLeft();
                }
                
                if (shiftY < blankY) {
                    currentTile.shiftDown();
                }
                
                if (shiftY > blankY) {
                    currentTile.shiftUp();
                }
                
                // now place the tile back into the tilearray
                vtiles.set(i, currentTile);
                
                //update the blank tile
                blankX = shiftX;
                blankY = shiftY;
            
            }
            
            
                    
        }
        
        
    }
    
    void thisWindowClosing(java.awt.event.WindowEvent e) {
        
        // tell our main loop to stop
        running = false;
        
        // dispose of our objects and quit
        System.out.println ("quit");
        dispose();
        ps.killSession();
        System.exit(5);
    }
}

class videoTile {
    // the video tile is very simple, just hold
    // info on the tile source, the current position
    // and the size
    
    int xTileSource;
    int yTileSource;
    int xTilePosition;
    int yTilePosition;
    int tileSize;
    
    videoTile(int _x, int _y, int _tileSize) {
        xTileSource = _x;
        yTileSource = _y;
        xTilePosition = _x;
        yTilePosition = _y;
        tileSize = _tileSize;
    }
    
    void shiftLeft() {
 
        xTilePosition--;
    }
    
    void shiftRight() {
        xTilePosition++;
    }
    
    void shiftUp() {
        yTilePosition--;
    }
    
    void shiftDown() {
        yTilePosition++;
    }
    
    int getSourceX() {
        return xTileSource * tileSize;
    }
    
    int getSourceY() {
        return yTileSource * tileSize;
    }
    
    int getDestX() {
        return xTilePosition * tileSize;
    }
    
    int getDestY() {
        return yTilePosition * tileSize;
    }
    
}
    